@@ -3,15 +3,17 @@ import math
 from functools import partial
 
 import numpy as np
-from glue.external.qt import QtGui
+
+from qtpy import QtWidgets
 
 from glue.utils.qt.widget_properties import CurrentComboProperty, FloatLineProperty, connect_bool_button, ButtonProperty
 from glue.utils.qt import load_ui
+from glue.utils import nonpartial
 
 __all__ = ["VispyOptionsWidget"]
 
 
-class VispyOptionsWidget(QtGui.QWidget):
+class VispyOptionsWidget(QtWidgets.QWidget):
 
     x_att = CurrentComboProperty('ui.combo_x_attribute')
     x_min = FloatLineProperty('ui.value_x_min')
@@ -30,6 +32,8 @@ class VispyOptionsWidget(QtGui.QWidget):
 
     visible_box = ButtonProperty('ui.checkbox_axes')
     perspective_view = ButtonProperty('ui.checkbox_perspective')
+    clip_data = ButtonProperty('ui.checkbox_clip')
+    native_aspect = ButtonProperty('ui.checkbox_native_aspect')
 
     def __init__(self, parent=None, vispy_widget=None, data_viewer=None):
 
@@ -50,6 +54,8 @@ class VispyOptionsWidget(QtGui.QWidget):
                                self.ui.value_y_stretch,
                                self.ui.value_z_stretch]
 
+        self.aspect = np.array([1, 1, 1], dtype=float)
+
         self._event_lock = False
 
         for slider, label in zip(self.stretch_sliders, self.stretch_values):
@@ -61,11 +67,17 @@ class VispyOptionsWidget(QtGui.QWidget):
 
         connect_bool_button(self._vispy_widget, 'visible_axes', self.ui.checkbox_axes)
         connect_bool_button(self._vispy_widget, 'perspective_view', self.ui.checkbox_perspective)
+        connect_bool_button(self._vispy_widget, 'clip_data', self.ui.checkbox_clip)
+
+        self.ui.checkbox_native_aspect.toggled.connect(self._update_aspect)
 
         if self._data_viewer is not None:
             self.ui.combo_x_attribute.currentIndexChanged.connect(self._data_viewer._update_attributes)
             self.ui.combo_y_attribute.currentIndexChanged.connect(self._data_viewer._update_attributes)
             self.ui.combo_z_attribute.currentIndexChanged.connect(self._data_viewer._update_attributes)
+            self.ui.combo_x_attribute.currentIndexChanged.connect(nonpartial(self._vispy_widget._update_attributes))
+            self.ui.combo_y_attribute.currentIndexChanged.connect(nonpartial(self._vispy_widget._update_attributes))
+            self.ui.combo_z_attribute.currentIndexChanged.connect(nonpartial(self._vispy_widget._update_attributes))
 
         self.ui.combo_x_attribute.currentIndexChanged.connect(self._update_attribute_limits)
         self.ui.combo_y_attribute.currentIndexChanged.connect(self._update_attribute_limits)
@@ -92,6 +104,17 @@ class VispyOptionsWidget(QtGui.QWidget):
 
         self._first_attributes = True
 
+    def _update_aspect(self):
+        if self.native_aspect:
+            self.aspect[0] = 1.
+            self.aspect[1] = (self.y_max - self.y_min) / (self.x_max - self.x_min)
+            self.aspect[2] = (self.z_max - self.z_min) / (self.x_max - self.x_min)
+            self.aspect /= self.aspect.max()
+        else:
+            self.aspect[:] = 1
+        self._vispy_widget._update_limits()
+        self._update_stretch()
+
     def set_limits(self, x_min, x_max, y_min, y_max, z_min, z_max):
 
         self._set_limits_enabled(False)
@@ -177,6 +200,28 @@ class VispyOptionsWidget(QtGui.QWidget):
             self._set_attributes_enabled(True)
             self._first_attributes = False
 
+        self._vispy_widget._update_attributes()
+        self._update_attribute_limits()
+
+    def _update_attributes_from_data_pixel(self, data):
+
+        z_cid, y_cid, x_cid = data.pixel_component_ids
+
+        self.ui.combo_x_attribute.clear()
+        self.ui.combo_x_attribute.addItem(x_cid.label, userData=x_cid)
+        self._components[x_cid] = data.get_component(x_cid)
+
+        self.ui.combo_y_attribute.clear()
+        self.ui.combo_y_attribute.addItem(y_cid.label, userData=y_cid)
+        self._components[y_cid] = data.get_component(y_cid)
+
+        self.ui.combo_z_attribute.clear()
+        self.ui.combo_z_attribute.addItem(z_cid.label, userData=z_cid)
+        self._components[z_cid] = data.get_component(z_cid)
+
+        self._set_attributes_enabled(True)
+
+        self._vispy_widget._update_attributes()
         self._update_attribute_limits()
 
     def _update_attribute_limits(self):
@@ -204,6 +249,7 @@ class VispyOptionsWidget(QtGui.QWidget):
         self._set_limits_enabled(True)
 
         self.ui.value_x_min.editingFinished.emit()
+        self._update_aspect()
 
     def _update_limits(self):
 
@@ -214,12 +260,13 @@ class VispyOptionsWidget(QtGui.QWidget):
         self._limits[self.y_att] = self.y_min, self.y_max
         self._limits[self.z_att] = self.z_min, self.z_max
 
-        self._vispy_widget._update_limits()
+        # The following calls self._vispy_widget._update_limits()
+        self._update_aspect()
 
     def _update_stretch(self):
-        self._vispy_widget._update_stretch(self.x_stretch,
-                                           self.y_stretch,
-                                           self.z_stretch)
+        self._vispy_widget._update_stretch(self.x_stretch * self.aspect[0],
+                                           self.y_stretch * self.aspect[1],
+                                           self.z_stretch * self.aspect[2])
 
     def _update_labels_from_sliders(self, label, slider):
         if self._event_lock:
