@@ -5,12 +5,18 @@ except ImportError:
 
 from glue.core import message as msg
 from glue.core import Data
+from glue.external.echo import add_callback
+from glue.utils import nonpartial
+
+from qtpy import QtWidgets
 
 from .vispy_widget import VispyWidget
 from .viewer_options import VispyOptionsWidget
+from .toolbar import VispyDataViewerToolbar
 
 
 class BaseVispyViewer(DataViewer):
+    _toolbar_cls = VispyDataViewerToolbar
 
     def __init__(self, session, parent=None):
 
@@ -24,6 +30,12 @@ class BaseVispyViewer(DataViewer):
         toolbar = self._toolbar_cls(vispy_widget=self._vispy_widget, parent=self)
         self.addToolBar(toolbar)
 
+        add_callback(self._vispy_widget, 'clip_data', nonpartial(self._toggle_clip))
+        add_callback(self._vispy_widget, 'clip_limits', nonpartial(self._toggle_clip))
+
+        self.status_label = None
+        self.client = None
+
     def register_to_hub(self, hub):
 
         super(BaseVispyViewer, self).register_to_hub(hub)
@@ -50,6 +62,10 @@ class BaseVispyViewer(DataViewer):
                       handler=self.update_window_title,
                       filter=has_data)
 
+        hub.subscribe(self, msg.NumericalDataChangedMessage,
+                      handler=self._numerical_data_changed,
+                      filter=has_data)
+
         hub.subscribe(self, msg.ComponentsChangedMessage,
                       handler=self._update_data,
                       filter=has_data)
@@ -77,6 +93,13 @@ class BaseVispyViewer(DataViewer):
     def data(self, data):
         self._data = data
 
+    # instance by object viewers later
+    def add_data(self, data):
+        return True
+
+    def _add_subset(self, message):
+        pass
+
     def _update_subset(self, message):
         if message.subset in self._layer_artist_container:
             for layer_artist in self._layer_artist_container[message.subset]:
@@ -93,6 +116,9 @@ class BaseVispyViewer(DataViewer):
             for layer_artist in self._layer_artist_container[message.data]:
                 layer_artist._update_data()
 
+    def _numerical_data_changed(self, message):
+        return self._update_data(message)
+
     def _redraw(self):
         self._vispy_widget.canvas.render()
 
@@ -117,13 +143,50 @@ class BaseVispyViewer(DataViewer):
         viewer = super(BaseVispyViewer, cls).__setgluestate__(rec, context)
 
         from ..scatter.layer_artist import ScatterLayerArtist
+        from ..volume.layer_artist import VolumeLayerArtist
 
         for layer_artist in viewer.layers:
-            if isinstance(layer_artist, ScatterLayerArtist) and isinstance(layer_artist.layer, Data):
-                viewer._options_widget._update_attributes_from_data(layer_artist.layer)
+            if isinstance(layer_artist.layer, Data):
+                if isinstance(layer_artist, ScatterLayerArtist):
+                    viewer._options_widget._update_attributes_from_data(layer_artist.layer)
+                elif isinstance(layer_artist, VolumeLayerArtist):
+                    viewer._options_widget._update_attributes_from_data_pixel(layer_artist.layer)
 
         for attr in sorted(rec['options'], key=lambda x: 0 if 'att' in x else 1):
             value = rec['options'][attr]
             setattr(viewer._options_widget, attr, context.object(value))
 
         return viewer
+
+    def show_status(self, text):
+        if not self.status_label:
+            statusbar = self.statusBar()
+            self.status_label = QtWidgets.QLabel()
+            statusbar.addWidget(self.status_label)
+        self.status_label.setText(text)
+
+    def initialize_toolbar(self):
+        # TODO: override this until we actually implement the toolbar properly
+        pass
+
+    def _update_attributes(self, index=None, layer_artist=None):
+
+        if layer_artist is None:
+            layer_artists = self._layer_artist_container
+        else:
+            layer_artists = [layer_artist]
+
+        for artist in layer_artists:
+            artist.set_coordinates(self._options_widget.x_att,
+                                   self._options_widget.y_att,
+                                   self._options_widget.z_att)
+
+    def restore_layers(self, layers, context):
+        pass
+
+    def _toggle_clip(self):
+        for layer_artist in self._layer_artist_container:
+            if self._vispy_widget.clip_data:
+                layer_artist.set_clip(self._vispy_widget.clip_limits)
+            else:
+                layer_artist.set_clip(None)
