@@ -4,11 +4,14 @@ import sys
 
 import numpy as np
 from ..extern.vispy import scene
-from ..extern.vispy.geometry import create_cube
+from .axes import AxesVisual3D
+
+from qtpy import QtWidgets
+from glue.utils.qt import get_qapp
 
 from glue.config import settings
 from glue.external.echo import CallbackProperty, add_callback
-from glue.external.qt import QtGui, get_qapp
+
 from glue.utils import nonpartial
 
 from matplotlib.colors import ColorConverter
@@ -16,10 +19,12 @@ from matplotlib.colors import ColorConverter
 rgb = ColorConverter().to_rgb
 
 
-class VispyWidget(QtGui.QWidget):
+class VispyWidget(QtWidgets.QWidget):
 
     visible_axes = CallbackProperty()
     perspective_view = CallbackProperty()
+    clip_data = CallbackProperty()
+    clip_limits = CallbackProperty()
 
     def _update_appearance_from_settings(self):
         self.canvas.bgcolor = rgb(settings.BACKGROUND_COLOR)
@@ -31,7 +36,7 @@ class VispyWidget(QtGui.QWidget):
 
         # Prepare Vispy canvas. We set the depth_size to 24 to avoid issues
         # with isosurfaces on MacOS X
-        self.canvas = scene.SceneCanvas(keys='interactive', show=False,
+        self.canvas = scene.SceneCanvas(keys=None, show=False,
                                         config={'depth_size': 24}, bgcolor=rgb(settings.BACKGROUND_COLOR))
 
         # Set up a viewbox
@@ -46,25 +51,26 @@ class VispyWidget(QtGui.QWidget):
         self.scene_transform = scene.STTransform()
         self.limit_transforms = {}
 
-        # Add a 3D cube to show us the unit cube. The 1.001 factor is to make
-        # sure that the grid lines are not 'hidden' by volume renderings on the
-        # front side due to numerical precision.
-        vertices, filled_indices, outline_indices = create_cube()
-        self.axis = scene.visuals.Mesh(vertices['position'],
-                                       outline_indices,
-                                       color=rgb(settings.FOREGROUND_COLOR), mode='lines')
-        self.axis.transform = self.scene_transform
-        self.view.add(self.axis)
+        fc = rgb(settings.FOREGROUND_COLOR)
+
+        self.axis = AxesVisual3D(axis_color=fc, tick_color=fc, text_color=fc,
+                                 tick_width=1, minor_tick_length=2,
+                                 major_tick_length=4, axis_width=0,
+                                 tick_label_margin=10, axis_label_margin=25,
+                                 tick_font_size=6, axis_font_size=8,
+                                 view=self.view,
+                                 transform=self.scene_transform)
 
         # Create a turntable camera. For now, this is the only camerate type
         # we support, but if we support more in future, we should implement
         # that here
 
-        # Remove the fov=60 here to solve the mismatch of selection problem
-        self.view.camera = scene.cameras.TurntableCamera(parent=self.view.scene, fov=60, distance=2.0)
+        # Orthographic perspective view as default
+        self.view.camera = scene.cameras.TurntableCamera(parent=self.view.scene,
+                                                         fov=0., distance=4.0)
 
         # Add the native canvas widget to this widget
-        layout = QtGui.QVBoxLayout()
+        layout = QtWidgets.QVBoxLayout()
         layout.setContentsMargins(0, 0, 0, 0)
         layout.addWidget(self.canvas.native)
         self.setLayout(layout)
@@ -86,15 +92,24 @@ class VispyWidget(QtGui.QWidget):
 
     def _toggle_perspective(self):
         if self.perspective_view:
-            self.view.camera.fov = 60
+            self.view.camera.fov = 30
+            self.axis.tick_font_size = 28
+            self.axis.axis_font_size = 35
         else:
             self.view.camera.fov = 0
+            self.axis.tick_font_size = 6
+            self.axis.axis_font_size = 8
 
     def add_data_visual(self, visual):
         self.limit_transforms[visual] = scene.STTransform()
         visual.transform = self.limit_transforms[visual]
         self.view.add(visual)
 
+    def _update_attributes(self):
+        self.axis.xlabel = self.options.x_att.label
+        self.axis.ylabel = self.options.y_att.label
+        self.axis.zlabel = self.options.z_att.label
+
     def _update_stretch(self, *stretch):
         self.scene_transform.scale = stretch
         self._update_limits()
@@ -110,9 +125,9 @@ class VispyWidget(QtGui.QWidget):
             raise Exception("We should never get here because if any data is "
                             "present, the limits should be set")
 
-        scale = [2 / (self.options.x_max - self.options.x_min) * self.options.x_stretch,
-                 2 / (self.options.y_max - self.options.y_min) * self.options.y_stretch,
-                 2 / (self.options.z_max - self.options.z_min) * self.options.z_stretch]
+        scale = [2 / (self.options.x_max - self.options.x_min) * self.options.x_stretch * self.options.aspect[0],
+                 2 / (self.options.y_max - self.options.y_min) * self.options.y_stretch * self.options.aspect[1],
+                 2 / (self.options.z_max - self.options.z_min) * self.options.z_stretch * self.options.aspect[2]]
 
         translate = [-0.5 * (self.options.x_min + self.options.x_max) * scale[0],
                      -0.5 * (self.options.y_min + self.options.y_max) * scale[1],
@@ -122,8 +137,18 @@ class VispyWidget(QtGui.QWidget):
             self.limit_transforms[visual].scale = scale
             self.limit_transforms[visual].translate = translate
 
+        self.axis.xlim = self.options.x_min, self.options.x_max
+        self.axis.ylim = self.options.y_min, self.options.y_max
+        self.axis.zlim = self.options.z_min, self.options.z_max
+
+        self.clip_limits = (self.options.x_min, self.options.x_max,
+                            self.options.y_min, self.options.y_max,
+                            self.options.z_min, self.options.z_max)
+
     def _reset_view(self):
         self.view.camera.reset()
+        # update the cam.fov with checkbox
+        self._toggle_perspective()
 
 
 if __name__ == "__main__":
