@@ -158,8 +158,8 @@ class LineVisual(CompoundVisual):
             The color to use when drawing the line. If an array is given, it
             must be of shape (..., 4) and provide one rgba color per vertex.
         width:
-            The width of the line in px. Line widths > 1px are only
-            guaranteed to work when using 'agg' method.
+            The width of the line in px. Line widths < 1 px will be rounded up
+            to 1 px when using the 'gl' method.
         connect : str or array
             Determines which vertices are connected by lines.
             * "strip" causes the line to be drawn with each vertex
@@ -346,14 +346,19 @@ class _GLLineVisual(Visual):
                 pass
 
         # Turn on line smooth and/or line width
-        if GL:
-            if self._parent._antialias:
-                GL.glEnable(GL.GL_LINE_SMOOTH)
-            else:
-                GL.glDisable(GL.GL_LINE_SMOOTH)
-            px_scale = self.transforms.pixel_scale
-            width = px_scale * self._parent._width
-            GL.glLineWidth(max(width, 1.))
+        try:
+            if GL:
+                if self._parent._antialias:
+                    GL.glEnable(GL.GL_LINE_SMOOTH)
+                else:
+                    GL.glDisable(GL.GL_LINE_SMOOTH)
+                px_scale = self.transforms.pixel_scale
+                width = px_scale * self._parent._width
+                GL.glLineWidth(max(width, 1.))
+        except Exception:
+            # FIXME: on AppVeyor, the above code doesn't run properly, possibly
+            #        because some of these OpenGL methods have been deprecated
+            pass
 
         if self._parent._changed['connect']:
             self._connect = self._parent._interpret_connect()
