@@ -92,6 +92,9 @@ class ScatterLayerArtist(LayerArtistBase):
         self._color_data = None
         self._size_data = None
 
+        self._clip_limits = None
+
+
     @property
     def visual(self):
         return self._multiscat
@@ -144,10 +147,10 @@ class ScatterLayerArtist(LayerArtistBase):
         elif self.size_mode == 'fixed':
             self._multiscat.set_size(self.id, self.size * self.size_scaling)
         else:
-            data = self.layer[self.size_attribute]
-            size = np.abs(np.nan_to_num(data))
-            size = 20 * (size - self.size_vmin) / (self.size_vmax - self.size_vmin)
+            data = self.layer[self.size_attribute].ravel()
+            size = 20 * (data - self.size_vmin) / (self.size_vmax - self.size_vmin)
             size_data = size * self.size_scaling
+            size_data[np.isnan(data)] = 0.
             self._multiscat.set_size(self.id, size_data)
 
     def _update_colors(self):
@@ -156,10 +159,10 @@ class ScatterLayerArtist(LayerArtistBase):
         elif self.color_mode == 'fixed':
             self._multiscat.set_color(self.id, self.color)
         else:
-            data = self.layer[self.cmap_attribute]
-            cmap_data = np.abs(np.nan_to_num(data))
-            cmap_data = (cmap_data - self.cmap_vmin) / (self.cmap_vmax - self.cmap_vmin)
+            data = self.layer[self.cmap_attribute].ravel()
+            cmap_data = (data - self.cmap_vmin) / (self.cmap_vmax - self.cmap_vmin)
             cmap_data = self.cmap(cmap_data)
+            cmap_data[:,3][np.isnan(data)] = 0.
             self._multiscat.set_color(self.id, cmap_data)
 
     def _update_alpha(self):
@@ -174,9 +177,9 @@ class ScatterLayerArtist(LayerArtistBase):
     def _update_data(self):
 
         try:
-            x = self.layer[self._x_coord]
-            y = self.layer[self._y_coord]
-            z = self.layer[self._z_coord]
+            x = self.layer[self._x_coord].ravel()
+            y = self.layer[self._y_coord].ravel()
+            z = self.layer[self._z_coord].ravel()
         except (IncompatibleAttribute, IndexError):
             # The following includes a call to self.clear()
             self.disable_invalid_attributes(self._x_coord, self._y_coord, self._z_coord)
@@ -184,6 +187,11 @@ class ScatterLayerArtist(LayerArtistBase):
         else:
             self._enabled = True
 
+        if self._clip_limits is not None:
+            xmin, xmax, ymin, ymax, zmin, zmax = self._clip_limits
+            keep = (x >= xmin) & (x <= xmax) & (y >= ymin) & (y <= ymax) & (z >= zmin) & (z <= zmax)
+            x, y, z = x[keep], y[keep], z[keep]
+
         self._marker_data = np.array([x, y, z]).transpose()
 
         # We need to make sure we update the sizes and colors in case
@@ -224,3 +232,7 @@ class ScatterLayerArtist(LayerArtistBase):
 
         for attr in sorted(kwargs, key=priorities):
             setattr(self, attr, kwargs[attr])
+
+    def set_clip(self, limits):
+        self._clip_limits = limits
+        self._update_data()
