@@ -1,12 +1,13 @@
-from vispy.color import BaseColormap
+from ..extern.vispy.color import BaseColormap
+
 
 def get_translucent_cmap(r, g, b):
-    
+
     class TranslucentCmap(BaseColormap):
         glsl_map = """
         vec4 translucent_fire(float t) {{
             return vec4({0}, {1}, {2}, t);
         }}
         """.format(r, g, b)
-        
-    return TranslucentCmap()
\ No newline at end of file
+
+    return TranslucentCmap()
